STRINGTABLE
BEGIN
  0, "Print the effective system id for the public id %1."
  1, "Print the effective system id for system id %1."
  2, "Print the effective system id for parameter entity %1."
  3, "Print the effective system id for the doctype %1."
  4, "Print the effective system id for the linktype %1."
  5, "Print the effective system id for entity %1."
  6, "Print the effective system id for notation %1."
  7, "Print the effective system id for the SGML declaration for doctype %1."
  8, "%1 prints effective system identifiers found in the catalogs on the"
  9, "standard output. It doesn't check that the system identifiers correspond"
  10, "to actual files. The return value is 0 if the last system identifier was"
  11, "successfully created and 1 otherwise."
  12, "LITERAL"
  13, "NAME"
END
