STRINGTABLE
BEGIN
  4000, "invalid option %1"
  4001, "missing argument for option %1"
  4002, "option %1 is ambiguous"
  4003, "option %1 doesn't allow an argument"
  4004, "Usage: %1"
  4005, "or: %1"
  4006, "%1 [OPTION] SYSID..."
  4007, "Short options need the same arguments as their long forms."
  4008, "%1 version %2"
  4009, "unknown BCTF %1"
  4010, "unknown encoding %1"
  4011, "cannot open output file %1 (%2)"
  4012, "cannot close output file %1 (%2)"
  4013, "Use bctf %1 for output."
  4014, "Use encoding %1 for output."
  4015, "Append error messages to file %1."
  4016, "Display the program version."
  4017, "Show this help text."
  4018, "NAME"
  4019, "FILE"
  4020, "NOTHING"
  4021, "Try the ""--help"" option for more information."
  4022, "ARG"
  4023, "Undocumented option (check the application's manual)."
END
