STRINGTABLE
BEGIN
  0, "unknown output option %1"
  1, "Enable batch mode."
  2, "Produce output according to %1."
  3, "Stop after the document prolog."
  4, "Suppress output."
  5, "Send RAST output to file %1."
  6, "Same as -wduplicate."
  7, "Same as -oline."
  8, "Same as -c."
  9, "Same as -wdefault."
  10, "Same as -wundef."
  11, "%1 parses and validates the SGML document whose document entity is"
  12, "specified by the system identifiers SYSID... and prints on the standard"
  13, "output a simple text representation of its Element Structure Information Set."
  14, "If more than one system identifier is specified, then the corresponding"
  15, "entities will be concatenated to form the document entity. If no system"
  16, "identifiers are specified, then %1 will read the document entity from"
  17, "the standard input. A command line system identifier of - can be used"
  18, "to refer to the standard input."
  19, "FILE"
  20, "OPTION"
  21, "SYSID"
END
